# vi: ts=4 expandtab
from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from awstrial.trial import util
import yaml


INSTANCE_STATUS = (
    ('reserved', _('reserved')),
    ('running', _('running')),
    ('shutdown', _('shutdown')),
)
EC2_REGIONS = (
    ('us-east-1', _('us-east-1')),
    ('us-west-1', _('us-west-1')),
    ('eu-west-1', _('eu-west-1')),
    ('ap-southeast-1', _('ap-southeast-1')),
)

class Campaign(models.Model):
    verbose_name = models.CharField(max_length=150, verbose_name=_('Name of the Campaign'))
    name = models.CharField(max_length=150, verbose_name=_('The slug, to use in the URL'))
    verbose_name = models.CharField(max_length=150, verbose_name=_('Name of the Campaign'))
    max_instances = models.IntegerField(verbose_name=_('Number of instances authorised'))
    active = models.BooleanField(verbose_name=_('Is this Campaign active?'))
    def __unicode__(self):
        return self.verbose_name
    def remaining(self):
        instance_count = Instances.objects.filter(campaign__verbose_name=self).count()
        remaining = self.max_instances-instance_count
        return remaining


class Instances(models.Model):
    instance_id = models.SlugField(_("Instance Identifier"), max_length=10, null=False, db_index=True)
    campaign = models.ForeignKey(Campaign)
    owner = models.ForeignKey(User)
    # for each of the time fields, the general flow is expected to be:
    #   pending -> running -> shutting-down -> terminated
    # if a state is found, and the previous state's time is not recorded
    # the update code will set them the same
    #   ie, if a instance is terminated, but there is no shutting down_time yet
    #   then set it to same as terminated
    reservation_time = models.DateTimeField(help_text=_('Time the instance was requested'), verbose_name=_('Spawn'))
    running_time = models.DateTimeField(help_text=_('First time the instance was found to be in running or later state'), verbose_name=_('Run time'), blank=True, null=True)
    shutdown_time = models.DateTimeField(help_text=_('First time the instance was found shutting-down or terminated'), verbose_name=_('Shutting-down time'), blank=True, null=True)
    terminated_time = models.DateTimeField(help_text=_('Time the instance was found terminated'), verbose_name=_('Terminated time'), blank=True, null=True)
    ph_time = models.DateTimeField(verbose_name=_('Phone-home Time'), blank=True, null=True)
    region = models.CharField(verbose_name=_('EC2 Region'), max_length=16, choices=EC2_REGIONS)
    ami_id = models.SlugField(_("AMI Identifier"), max_length=12, null=False, db_index=False)
    console_start = models.TextField(verbose_name=_('Console Output at start'), blank=True, null=True)
    console_end = models.TextField(verbose_name=_('Console Output after terminated'), blank=True, null=True)
    hostname = models.CharField(verbose_name=_('EC2 Hostname'), max_length=128, blank=True)
    ip = models.CharField(verbose_name=_('IP address'), max_length=16, blank=True)
    pubkeys = models.TextField(verbose_name=_('instance public ssh keys'), blank=True, null=True)
    secret = models.CharField(help_text=_('This is a randomly generated secret shared between instance and server.  Should be one-time use only'), verbose_name=_('Secret Key'), max_length=32, blank=True)
    config_info = models.TextField(verbose_name=_('Configuration info'), blank=True, null=True)

    def __unicode__(self):
        return self.instance_id

    def status(self):
        # return the state based on the timestamps that are set
        # one of 'running', 'shutting-down'
        if self.terminated_time: return('terminated')
        if self.shutdown_time: return('shutting-down')
        if self.running_time: return('running')
        if self.reservation_time: return('pending')

    def olderThan(self, td, dt=None):
        # expects a time delta
        if not dt:
            dt = util.dtnow()
        return(self.reservation_time + td < dt)

    def _cfgval_get(self,name,default=None):
        if not self.config_info: self.config_info = ""
        c = yaml.load(self.config_info)
        if not isinstance(c,dict): c = { }
        if name in c: return c[name]
        return default

    def _cfgval_set(self,name,value):
        if not self.config_info: self.config_info = ""
        c = yaml.load(self.config_info)
        if not isinstance(c,dict): c = { }
        c[name]=value
        self.config_info = c

    @property
    def byobu(self):
        return(self._cfgval_get("byobu",False))

    @byobu.setter
    def byobu(self,val):
        self._cfgval_set("byobu",val)

    @property
    def config(self):
        return(self._cfgval_get("config",None))

    @config.setter
    def config(self,val):
        self._cfgval_set("config",val)

class Feedback(models.Model):
    user = models.ForeignKey(User)
    comment = models.TextField(verbose_name=_('Feedback / Comments'))

    def __unicode__(self):
        return str(self.user)

